# Navigation_Client.py
#
# Copyright 2019 Ryota Higashi
# This software may be modified and distributed under the terms
# of the MIT license
#
# For python3
# For HRI Engine

"""Navigation_Client
"""

import threading
# import logging
import xmlrpc.client

from pyrois import RoIS_Common, RoIS_HRI


class Command(RoIS_Common.Command):
    """Command
    """
    def __init__(self, uri):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)

    def start(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.start())
        return status

    def stop(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.stop())
        return status

    def suspend(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.suspend())
        return status

    def resume(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.resume())
        return status

    def set_parameter(self, target_position, time_limit, routing_policy):
        s = self._proxy.set_parameter(target_position, time_limit, routing_policy)
        status = RoIS_HRI.ReturnCode_t(s)
        return status
    

class Query(RoIS_Common.Query):
    """Query
    """
    def __init__(self, uri):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)

    def component_status(self):
        (status, c_status) = self._proxy.component_status()
        return (RoIS_HRI.ReturnCode_t(status), RoIS_Common.Component_Status(c_status))
    
    def get_parameter(self):
        (status, target_position, time_limit, routing_policy) = self._proxy.get_parameter()
        return (RoIS_HRI.ReturnCode_t(status), target_position, time_limit, routing_policy)


class Event(RoIS_Common.Event):
    """Event
    """
    def __init__(self, uri):
        self._uri = uri
        self._e_proxy = xmlrpc.client.ServerProxy(self._uri)
        self.events = []
        # if logger is not None:
        #     self.logger = logger


class Navigation_Client(Command, Query, Event):
    """Navigation_Client
    """
    def __init__(self, uri):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)
        self._e_proxy = xmlrpc.client.ServerProxy(self._uri)
        self.events = []


# Sound_Detection.py
#
# Copyright 2019 Ryota Higashi
# This software may be modified and distributed under the terms
# of the MIT license
#
# For python3
# For HRI Component

"""Sound_Detection
"""

import sys
import queue
import time
from datetime import datetime
import threading

from pyrois import RoIS_Common, RoIS_HRI

import socketserver
import xmlrpc.server

class ThreadingXMLRPCServer(socketserver.ThreadingMixIn, xmlrpc.server.SimpleXMLRPCServer):
    """ThreadingXMLRPCServer
    """
    pass


class Command(RoIS_Common.Command):
    """Command
    """
    def __init__(self, c):
        self._component = c

    def start(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def stop(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def suspend(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def resume(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status


class Query(RoIS_Common.Query):
    """Query
    """
    def __init__(self, c):
        self._component = c

    def component_status(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        c_status = self.c_status
        return (status, c_status)


class Event(RoIS_Common.Event):
    """Event
    """
    def __init__(self, c):
        self._component = c
        self.event_queue = queue.Queue()

    def poll_event(self):
        """poll_event
        """
        msg = self.event_queue.get()
        return msg

    def sound_detected(self, timestamp, number):
        """sound_detected
        """
        msg = xmlrpc.client.dumps((timestamp, number), 'sound_detected')
        self.event_queue.put(msg)


class Sound_Detection(Event, Command, Query):
    """Sound_Detection
    """
    def __init__(self, c):
        super().__init__(c)
        self._component = c
        self.c_status = RoIS_Common.Component_Status.READY.value


class component:
    """component
    """
    def __init__(self):
        self._state = False


def event_dispatch(sd):
    """event_dispatch
    """
    sd.sound_detected(datetime.now().isoformat(), 1)
    time.sleep(0.1)
    sd.sound_detected(datetime.now().isoformat(), 1)


def example_sd(port):
    """example_sd
    """
    c = component()
    sd = Sound_Detection(c)

    # start the timer to dispatch events
    t = threading.Timer(0.1, event_dispatch, args=(sd,))
    t.start()

    # start the XML-RPC server
    server = ThreadingXMLRPCServer(("0.0.0.0", port), logRequests=False)
    server.register_instance(sd)
    server.register_introspection_functions()
    server.register_multicall_functions()
    server.serve_forever()


if __name__ == '__main__':
    example_sd(8007)



import time
import threading
from multiprocessing import Process, Queue
import unittest
import http
import xmlrpc.client
import logging
import logging.handlers
from datetime import datetime

from pyrois import RoIS_HRI, RoIS_Common, RoIS_Service
from pyrois import Service_Application_IF, Service_Application_Base_sample
from pyrois import HRI_Engine_client, HRI_Engine_sample
from pyrois import Person_Detection_client, Person_Detection
from pyrois import IF_server

def test_engine(port):
    """test_engine
    """
    IF_server.IF_server(port).run(HRI_Engine_sample.IF(HRI_Engine_sample.MyHRIE()))

def event_dispatch(sa):
    """event_dispatch
    """
    sa.completed("0", RoIS_Service.Completed_Status.OK.value)
    time.sleep(0.1)
    sa.notify_error("1", RoIS_Service.ErrorType.ENGINE_INTERNAL_ERROR.value)
    time.sleep(0.1)
    sa.notify_event("2", "sensor", "0", "2100-01-01T00:00:01+09:00")

# def event_dispatch_long(sa):
#     sa.completed("0", RoIS_Service.Completed_Status.OK.value)
#     time.sleep(1)
#     sa.notify_error("0", RoIS_Service.ErrorType.ENGINE_INTERNAL_ERROR.value)
#     time.sleep(60)
#     sa.notify_event("0", "sensor", "0", "2100-01-01T00:00:01+09:00")

def sample_sa(port):
    """sample_sa
    """
    sa = Service_Application_Base_sample.Service_Application_Base()

    # start the timer to dispatch events
    t = threading.Timer(0.1, event_dispatch, args=(sa,))
    t.start()

    # start the XML-RPC server
    IF_server.IF_server(port).run(sa)

def sample_sa_IF(url, q):
    try:
        logger = logging.getLogger('Service_Application_IF')
        logger.setLevel(logging.DEBUG)
        ch = logging.handlers.QueueHandler(q)
        # ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)
        formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        logger.addHandler(ch)
        a = Service_Application_IF.Service_Application_IF(url, logger=logger)
        while True:
            time.sleep(5)
        print("Finished")
    except KeyboardInterrupt:
        print("Interrupted")

def pd_event_dispatch(pd):
    """event_dispatch
    """
    pd.person_detected(datetime.now().isoformat(), 1)
    time.sleep(0.1)
    pd.person_detected(datetime.now().isoformat(), 1)


def sample_pd(port):
    """sample_pd
    """
    c = Person_Detection.component()
    pd = Person_Detection.Person_Detection(c)

    # start the timer to dispatch events
    t = threading.Timer(0.1, pd_event_dispatch, args=(pd,))
    t.start()

    # start the XML-RPC server
    IF_server.IF_server(port).run(pd)


# System_Information.py
#
# Copyright 2019 Ryota Higashi
# This software may be modified and distributed under the terms
# of the MIT license
#
# For python3
# For HRI Component

"""System_Information
"""

import sys
import queue
import time
from datetime import datetime
import threading

from pyrois import RoIS_Common, RoIS_HRI

import socketserver
import xmlrpc.server

class ThreadingXMLRPCServer(socketserver.ThreadingMixIn, xmlrpc.server.SimpleXMLRPCServer):
    """ThreadingXMLRPCServer
    """
    pass


class Query(RoIS_Common.Query):
    """Query
    """
    def __init__(self, c):
        self._component = c
    
    def robot_position(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        timestamp = "2100-01-01T00:00:01+09:00"
        robot_ref = ["urn:x-rois:def:HRIComponent:Kyutech::robot1"]  # codebook, version
        position_data = ["33°53′41″", "130°50′30″"]
        return(status, timestamp, robot_ref, position_data)

    def engine_status(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        e_status = RoIS_Common.Component_Status.READY.value
        operable_time = ["2100-01-01T00:00:01+09:00","2100-01-01T00:00:01+09:00"]
        return (status, e_status, operable_time)


class System_Information(Query):
    """System_Information
    """
    def __init__(self, c):
        self._component = c


class component:
    """component
    """
    def __init__(self):
        self._state = False


def event_dispatch(pd):
    """event_dispatch
    """

def example_si(port):
    """example_si
    """
    c = component()
    si = System_Information(c)

    # start the timer to dispatch events
    t = threading.Timer(0.1, event_dispatch, args=(si,))
    t.start()

    # start the XML-RPC server
    server = ThreadingXMLRPCServer(("0.0.0.0", port), logRequests=False)
    server.register_instance(si)
    server.register_introspection_functions()
    server.register_multicall_functions()
    server.serve_forever()


if __name__ == '__main__':
    example_si(8001)





# Component_Sample.py
# 
# Copyright 2019 Ryota Higashi

from pyrois import RoIS_Common
import time

class Component_Sample():
    def __init__(self):
        self.c_status = RoIS_Common.Component_Status.READY.value

    def Speech_Synthesis(self, speech_text, SSML_text, volume, languages, character):
        self.c_status = RoIS_Common.Component_Status.BUSY.value
        
        print("コンポーネントを実行")
        time.sleep(1)
        if speech_text is not "":
            print("Speech_Text:%s"%(speech_text))
            time.sleep(1)
            if SSML_text is not "":
                print("SSML_Text:%s"%(SSML_text))
                time.sleep(1)
                if volume is not "":
                    print("Volume:%s"%(volume))
                    time.sleep(1)
                    if languages is not "":
                        print("Language:%s"%(languages))
                        time.sleep(1)
                        if character is not "":
                            print("Character:%s"%(character))
                            self.c_status = RoIS_Common.Component_Status.READY.value
                        else:
                            print("Argument_of_[character]_does_not_be_specified")
                            self.c_status = RoIS_Common.Component_Status.ERROR.value
                    else:
                        print("Argument_of_[languages]_does_not_be_specified")
                        self.c_status = RoIS_Common.Component_Status.ERROR.value        
                else:
                    print("Argument_of_[volume]_does_not_be_specified")
                    self.c_status = RoIS_Common.Component_Status.ERROR.value
            else:
                print("Argument_of_[SSML_text]_does_not_be_specified")
                self.c_status = RoIS_Common.Component_Status.ERROR.value
        else:
            print("Argument_of_[speech_text]_does_not_be_specified")
            self.c_status = RoIS_Common.Component_Status.ERROR.value
        return(self.c_status)

# if __name__ == "__main__":
#     test1 = Component_Sample()
#     test1.moji()


# HRI_Engine_client.py
#
# Copyright 2017 Eiichi Inohira
# Copyright 2019 Ryota Higashi
# This software may be modified and distributed under the terms
# of the MIT license
#
# For python 3
# For Service Application

"""HRI_Engine_client
"""
import platform
from datetime import datetime
import time
import xmlrpc.client
import logging
import queue
import json

from pyrois import RoIS_HRI
from . import Service_Application_IF_sample

class SystemIF(RoIS_HRI.SystemIF):
    """SystemIF
    """
    def __init__(self, proxy_obj):
        self._proxy = proxy_obj
        self.logger.info("HRI_Engine_client_sample.SystemIF(): complete init")

    def connect(self):
        try:
            status = RoIS_HRI.ReturnCode_t(self._proxy.connect())
            self.logger.info("Connect:status[{}]".format(status))
        except Exception as e:
            status = RoIS_HRI.ReturnCode_t.ERROR
            self.logger.error("Connect:{}".format(e))
        return status

    def disconnect(self):
        try:
            status = RoIS_HRI.ReturnCode_t(self._proxy.disconnect())
            self.logger.info("Disconnect:status[{}]".format(status))
        except Exception as e:
            status = RoIS_HRI.ReturnCode_t.ERROR
            self.logger.error("Disconnect:{}".format(e))
        return status

    def get_profile(self, condition):
        try:
            (s, profile) = self._proxy.get_profile(condition)
            status = RoIS_HRI.ReturnCode_t(s)
            self.logger.info("Get_Profile:status[{}],profile[{}]".format(status,profile))
        except Exception as e:
            status = RoIS_HRI.ReturnCode_t.ERROR
            profile = ""
            self.logger.error("Get_Profile:{}".format(e))
        return (status, profile)

    def get_error_detail(self, error_id, condition):
        try:
            (s, results) = self._proxy.get_error_detail(error_id, condition)
            status = RoIS_HRI.ReturnCode_t(s)
            self.logger.info("Get_Error_Detail({}):status[{}]".format(error_id,status))
        except Exception as e:
            status = RoIS_HRI.ReturnCode_t.ERROR
            results = [""]
            self.logger.error("Get_Error_Detail({}):{}".format(error_id,e))
        return (status, results)


class CommandIF(RoIS_HRI.CommandIF):
    """CommandIF
    """
    def __init__(self, proxy_obj):
        self._proxy = proxy_obj
        self.logger.info("HRI_Engine_client_sample.CommandIF(): complete init")

    def search(self, condition):
        try:
            (s, component_ref_list) = self._proxy.search(condition)
            status = RoIS_HRI.ReturnCode_t(s)
            self.logger.info("Search:status[{}],component_ref[{}]".format(status,component_ref_list))
        except Exception as e:
            status = RoIS_HRI.ReturnCode_t.ERROR
            component_ref_list = [""]
            self.logger.error("Search:{}".format(e))
        return (status, component_ref_list)

    def bind(self, component_ref):
        try:
            status = RoIS_HRI.ReturnCode_t(self._proxy.bind(component_ref))
            self.logger.info("Bind({}):status[{}]".format(component_ref,status))
        except Exception as e:
            status = RoIS_HRI.ReturnCode_t.ERROR
            self.logger.error("Bind({}):{}".format(component_ref,e))
        return status

    def bind_any(self, condition):
        try:
            (s, component_ref_list) = self._proxy.search(condition)
            status = RoIS_HRI.ReturnCode_t(s)
            self.logger.info("Bind_Any:status[{}]".format(status))
        except Exception as e:
            status = RoIS_HRI.ReturnCode_t.ERROR
            component_ref_list = [""]
            self.logger.error("Bind_Any:{}".format(e))
        return (status, component_ref_list)

    def release(self, component_ref):
        try:
            status = RoIS_HRI.ReturnCode_t(self._proxy.release(component_ref))
            self.logger.info("Release({}):status[{}]".format(component_ref,status))
        except Exception as e:
            status = RoIS_HRI.ReturnCode_t.ERROR
            self.logger.error("Release({}):{}".format(component_ref,e))
        return status

    def get_parameter(self, component_ref):
        try:
            (s, parameter_list) = self._proxy.get_parameter(component_ref)
            status = RoIS_HRI.ReturnCode_t(s)
            self.logger.info("Get_Parameter({}):status[{}],parameter_list[{}]".format(component_ref,status,parameter_list))
        except Exception as e:
            status = RoIS_HRI.ReturnCode_t.ERROR
            parameter_list = [""]
            self.logger.error("Get_Parameter({}):{}".format(component_ref,e))
        return (status, parameter_list)

    def set_parameter(self, component_ref, parameters):
        try:
            (s, command_id) = self._proxy.set_parameter(component_ref, parameters)
            status = RoIS_HRI.ReturnCode_t(s)
            self.logger.info("Set_Parameter({}):status[{}],id[{}]".format(component_ref,status,command_id))
        except Exception as e:
            status = RoIS_HRI.ReturnCode_t.ERROR
            command_id = "00"
            self.logger.error("Set_Parameter({}):{}".format(component_ref,e))
        return (status, command_id)

    def execute(self, command_unit_list):
        try:
            j = json.dumps(command_unit_list, default=self.object_to_dict, indent=4)
            d = json.loads(j)
            if "command_unit_list" not in d.keys():
                status = RoIS_HRI.ReturnCode_t.BAD_PARAMETER
                return (status, command_unit_list)
            (s, command_unit_list) = self._proxy.execute(j)
            status = RoIS_HRI.ReturnCode_t(s)
            self.logger.info("Execute({}):status[{}]".format(command_unit_list,status))
        except Exception as e:
            status = RoIS_HRI.ReturnCode_t.ERROR
            self.logger.error("Execute({}):{}".format(command_unit_list,e))
        return (status, command_unit_list)

    def get_command_result(self, command_id, condition):
        try:
            (s, results) = self._proxy.get_command_result(command_id, condition)
            status = RoIS_HRI.ReturnCode_t(s)
            self.logger.info("Get_Command_Result({}):status[{}],results[{}]".format(command_id,status,results))
        except Exception as e:
            status = RoIS_HRI.ReturnCode_t.ERROR
            self.logger.error("Disconnect:{}".format(e))
        return (status, results)


class QueryIF(RoIS_HRI.QueryIF):
    """QueryIF
    """
    def __init__(self, proxy_obj):
        self._proxy = proxy_obj
        self.logger.info("HRI_Engine_client_sample.QueryIF(): complete init")

    def query(self, query_type, condition):
        try:
            (s, results) = self._proxy.query(query_type, condition)
            status = RoIS_HRI.ReturnCode_t(s)
            self.logger.info("Query({}):status[{}],results[{}]".format(query_type,status,results))
        except Exception as e:
            status = RoIS_HRI.ReturnCode_t.ERROR
            results = [""]
            self.logger.error("Query({}):{}".format(query_type,e))
        return (status, results)


class EventIF(RoIS_HRI.EventIF):
    """EventIF
    """
    def __init__(self, proxy_obj):
        self._proxy = proxy_obj
        self.logger.info("HRI_Engine_client_sample.EventIF(): complete init")

    def subscribe(self, event_type, condition):
        try:
            (s, subscribe_id) = self._proxy.subscribe(event_type, condition)
            status = RoIS_HRI.ReturnCode_t(s)
            self.logger.info("Subscribe({}):status[{}],id[{}]".format(event_type,status,subscribe_id))
        except Exception as e:
            status = RoIS_HRI.ReturnCode_t.ERROR
            subscribe_id = "00"
            self.logger.error("Subscribe({}):{}".format(event_type,e))
        return (status, subscribe_id)

    def unsubscribe(self, subscribe_id):
        try:
            s = self._proxy.unsubscribe(subscribe_id)
            status = RoIS_HRI.ReturnCode_t(s)
            self.logger.info("Unsubscribe({}):status[{}]".format(subscribe_id,status))
        except Exception as e:
            status = RoIS_HRI.ReturnCode_t.ERROR
            self.logger.error("Unsubscribe({}):{}".format(subscribe_id,e))
        return status

    def get_event_detail(self, event_id, condition):
        try:
            (s, results) = self._proxy.get_event_detail(event_id, condition)
            status = RoIS_HRI.ReturnCode_t(s)
            self.logger.info("Get_Event_Detail({}):status[{}]".format(event_id,status))
        except Exception as e:
            status = RoIS_HRI.ReturnCode_t.ERROR
            results = [""]
            self.logger.error("Get_Event_Detail({}):{}".format(event_id,e))
        return (status, results)


class IF(SystemIF, CommandIF, QueryIF, EventIF, Service_Application_IF_sample.Service_Application_IF):
    """IF
    """
    def __init__(self, uri, logger=None, event_queue=None):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)
        # self.a = Service_Application_Base_example.Service_Application_Base()

        self.dt = datetime.now()
        self.dt_format = self.dt.strftime("%Y%m%d-%H%M")
        self.os = platform.system()

        if logger is not None:
            self.logger = logger
        else:
            self.logger = logging.getLogger('Service_Application: HRI_Engine_client_sample.py')
            self.logger.setLevel(logging.DEBUG)
            formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
            #self.fh = logging.FileHandler('{}_{}.log'.format(self.dt_format,self.os))
            self.fh = logging.FileHandler('{}.log'.format(self.dt_format))
            self.fh.setLevel(logging.DEBUG)
            self.fh.setFormatter(formatter)
            self.logger.addHandler(self.fh)

            self.ch = logging.StreamHandler()
            self.ch.setLevel(logging.ERROR)
            self.ch.setFormatter(formatter)
            self.logger.addHandler(self.ch)

        import os
        self.logger.info("This_logger_has_also_logs_in_Service_Application_IF_sample.py {}".format(os.getpid()))

        Service_Application_IF_sample.Service_Application_IF.__init__(self,self._uri,logger=self.logger,event_queue=event_queue)
        self.logger.info("HRI_Engine_client_sample.IF(): complete init")
    
    def object_to_dict(self, item):
        if isinstance(item, object) and hasattr(item, '__dict__'):
            return item.__dict__
        else:
            raise TypeError

if __name__ == "__main__":
    uri = 'http://127.0.0.1:8017'
    test = IF(uri)


# test_main_sub_engine2.py
#
# Copyright 2020 Ryota Higashi
# Copyright 2021 Eiichi Inohira
#
# This software may be modified and distributed under the terms
# of the MIT license
#
# for python 3

"""unittest
"""

import time
from multiprocessing import Process, Queue
import unittest
import xmlrpc.client
import sys
import re
import logging
import platform
import atexit

from pyrois import RoIS_HRI, RoIS_Common, RoIS_Service
from pyrois_common import HRI_Engine_sample, HRI_Engine_client_sample


from pyrois_common import System_Information
from pyrois_common import Person_Detection
from pyrois_common import Person_Localization
from pyrois_common import Person_Identification
from pyrois_common import Face_Detection
from pyrois_common import Face_Localization
from pyrois_common import Sound_Detection
from pyrois_common import Sound_Localization
from pyrois_common import Speech_Recognition
from pyrois_common import Gesture_Recognition
from pyrois_common import Speech_Synthesis
from pyrois_common import Reaction
from pyrois_common import Navigation
from pyrois_common import Follow
from pyrois_common import Move


eng_profile = """<?xml version="1.0" encoding="UTF-8"?>
<!-->
Robotic Interaction Service Framework Version 1.2
<-->

<rois:HRIEngineProfile gml:id="engine_profile"
    xmlns:rois="http://www.omg.org/spec/RoIS/20151201"
    xmlns:gml="http://www.opengis.net/gml/3.2">
    <gml:identifier>urn:x-rois:def:HRIEngine:Kyutech::main</gml:identifier>
    <gml:name>MainHRI</gml:name>
    <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:main:PersonDetection</rois:HRIComponent>
    <rois:SubProfile>
        <gml:identifier>urn:x-rois:def:HRIEngine:Kyutech::sub01</gml:identifier>
        <gml:name>SubEng01</gml:name>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonIdentification</rois:HRIComponent>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis</rois:HRIComponent>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow</rois:HRIComponent>
    </rois:SubProfile>
</rois:HRIEngineProfile>"""

        
comp_profile = """<?xml version="1.0" encoding="UTF-8"?>

<rois:HRIComponentProfile
    gml:id="component_profile"
    xmlns:rois="http://www.omg.org/spec/RoIS/20151201"
    xmlns:ns2="http://www.w3.org/1999/xlink"
    xmlns:gml="http://www.opengis.net/gml/3.2"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
    <gml:identifier codeSpace="urn:ietf:rfc:2141">
        urn:x-rois:def:component:OMG::SpeechSynthesis</gml:identifier>
    <gml:name>speech_synthesizer</gml:name>
    
    <!-- ===== Sub Component Profiles ===== -->
    <rois:SubComponentProfile>
        urn:x-rois:def:Component:OMG::RoISCommon</rois:SubComponentProfile>
    
    <!-- ===== Query Messages ===== -->
    <rois:MessageProfile rois:name="synthesizable_languages"
        xsi:type="rois:QueryMessageProfileType">
        <rois:Results rois:description="list of available languages" rois:name="languages">
            <rois:data_type_ref rois:code="string[]"/>
        </rois:Results>
    </rois:MessageProfile>
    <rois:MessageProfile rois:name="available_voices"
        xsi:type="rois:QueryMessageProfileType">
        <rois:Results rois:description="list of available voice characters"
            rois:name="characters">
            <rois:data_type_ref rois:code="string[]"/>
        </rois:Results>
    </rois:MessageProfile>
    
    <!-- ===== Parameter Profiles ===== -->
    <rois:ParameterProfile rois:description="speech text in plain text"
        rois:name="speech_text">
        <rois:data_type_ref rois:code="string"/>
    </rois:ParameterProfile>
    <rois:ParameterProfile rois:description="speech text in SSML text"
        rois:name="ssml_text">
        <rois:data_type_ref rois:code="string"/>
    </rois:ParameterProfile>
    <rois:ParameterProfile rois:description="Volume" rois:name="volume"
        rois:default_value="50">
        <rois:data_type_ref rois:code="int"/>
    </rois:ParameterProfile>
    <rois:ParameterProfile rois:description="Language of the speech" rois:name="language"
        rois:default_value="en">
        <rois:data_type_ref rois:code="string"/>
    </rois:ParameterProfile>
    <rois:ParameterProfile rois:description="character of the voice" rois:name="character"
        rois:default_value="default">
        <rois:data_type_ref rois:code="string"/>
    </rois:ParameterProfile>
</rois:HRIComponentProfile>"""

right_event_result = {
    (('sub01-6', RoIS_Service.Completed_Status.OK), 'completed'), # completed
    (('sub01-7', RoIS_Service.Completed_Status.OK), 'completed'), # completed
    (('main-1', 'person_detected', '1'), 'notify_event'),
    (('main-2', 'person_detected', '1'), 'notify_event'),
    (('sub01-1', 'person_identified', '2'), 'notify_event'),
    (('sub01-2', 'person_identified', '2'), 'notify_event')
    }

right_command_result = [
    RoIS_HRI.ReturnCode_t.OK,
    
    (RoIS_HRI.ReturnCode_t.OK, eng_profile),
    
    # (RoIS_HRI.ReturnCode_t.OK, ['<?xml version="1.0" encoding="UTF-8"?>\n\n<rois:HRIComponentProfile\n    gml:id="component_profile"\n    xmlns:rois="http://www.omg.org/spec/RoIS/20151201"\n    xmlns:ns2="http://www.w3.org/1999/xlink"\n    xmlns:gml="http://www.opengis.net/gml/3.2"\n    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">\n    <gml:identifier codeSpace="urn:ietf:rfc:2141">\n        urn:x-rois:def:component:OMG::SpeechSynthesis</gml:identifier>\n    <gml:name>speech_synthesizer</gml:name>\n    \n    <!-- ===== Sub Component Profiles ===== -->\n    <rois:SubComponentProfile>\n        urn:x-rois:def:Component:OMG::RoISCommon</rois:SubComponentProfile>\n    \n    <!-- ===== Query Messages ===== -->\n    <rois:MessageProfile rois:name="synthesizable_languages"\n        xsi:type="rois:QueryMessageProfileType">\n        <rois:Results rois:description="list of available languages" rois:name="languages">\n            <rois:data_type_ref rois:code="string[]"/>\n        </rois:Results>\n    </rois:MessageProfile>\n    <rois:MessageProfile rois:name="available_voices"\n        xsi:type="rois:QueryMessageProfileType">\n        <rois:Results rois:description="list of available voice characters"\n            rois:name="characters">\n            <rois:data_type_ref rois:code="string[]"/>\n        </rois:Results>\n    </rois:MessageProfile>\n    \n    <!-- ===== Parameter Profiles ===== -->\n    <rois:ParameterProfile rois:description="speech text in plain text"\n        rois:name="speech_text">\n        <rois:data_type_ref rois:code="string"/>\n    </rois:ParameterProfile>\n    <rois:ParameterProfile rois:description="speech text in SSML text"\n        rois:name="ssml_text">\n        <rois:data_type_ref rois:code="string"/>\n    </rois:ParameterProfile>\n    <rois:ParameterProfile rois:description="Volume" rois:name="volume"\n        rois:default_value="50">\n        <rois:data_type_ref rois:code="int"/>\n    </rois:ParameterProfile>\n    <rois:ParameterProfile rois:description="Language of the speech" rois:name="language"\n        rois:default_value="en">\n        <rois:data_type_ref rois:code="string"/>\n    </rois:ParameterProfile>\n    <rois:ParameterProfile rois:description="character of the voice" rois:name="character"\n        rois:default_value="default">\n        <rois:data_type_ref rois:code="string"/>\n    </rois:ParameterProfile>\n</rois:HRIComponentProfile>']),
    (RoIS_HRI.ReturnCode_t.OK, [comp_profile]),

    (RoIS_HRI.ReturnCode_t.OK,['urn:x-rois:def:HRIComponent:Kyutech:main:PersonDetection',
    'urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonIdentification', 
    'urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis', 
    'urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow', 
    # 'urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceDetection', 
    # 'urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceLocalization', 
    # 'urn:x-rois:def:HRIComponent:Kyutech:sub02:Navigation', 
    # 'urn:x-rois:def:HRIComponent:Kyutech:sub03:PersonDetection', 
    # 'urn:x-rois:def:HRIComponent:Kyutech:sub03:Move'
    ]),

    RoIS_HRI.ReturnCode_t.OK, # bind:PersonDetection
    RoIS_HRI.ReturnCode_t.OK, # bind:PersonIdentification
    RoIS_HRI.ReturnCode_t.OK, # bind:SpeechSynthesis
    RoIS_HRI.ReturnCode_t.OK, # bind:Follow
    # RoIS_HRI.ReturnCode_t.OK, # bind:FaceDetection
    # RoIS_HRI.ReturnCode_t.OK, # bind:FaceLocalization
    # RoIS_HRI.ReturnCode_t.OK, # bind:Navigation
    # RoIS_HRI.ReturnCode_t.OK, # release:Follow
    (RoIS_HRI.ReturnCode_t.OK,['', '', 10, '', '', '', '']), # get_parameter:SpeechSynthesis
    (RoIS_HRI.ReturnCode_t.OK,["",1000,10]), # get_parameter:Follow
    (RoIS_HRI.ReturnCode_t.OK,"sub01-6"), # set_parameter:SpeechSynthesis
    (RoIS_HRI.ReturnCode_t.OK,"sub01-7"), # set_parameter:Follow
    (RoIS_HRI.ReturnCode_t.OK,"1"), # subscribe:PersonDetection
    (RoIS_HRI.ReturnCode_t.OK,"2"), # subscribe:PersonIdentification
    # RoIS_HRI.ReturnCode_t.OK, # unsubscribe
    # (RoIS_HRI.ReturnCode_t.OK,"2"), # subscribe
    (RoIS_HRI.ReturnCode_t.OK,["Bind(urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonIdentification):Success"]), # get_command_result
    RoIS_HRI.ReturnCode_t.OK # disconnect
    ]

def kill_ps(ps):
    # terminate the server process
    for x in ps:
        x.kill()
        time.sleep(0.5)

def test_sa(q):
    i = HRI_Engine_client_sample.IF("http://127.0.0.1:8016")
    res = [
        i.connect(),
        i.get_profile(""),
        i.query("SpeechSynthesis",""),
        i.search(""),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:main:PersonDetection'),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonIdentification'),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis'),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow'),
        # i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceDetection'),
        # i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceLocalization'),
        # i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub02:Navigation'),
        # i.release('urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow'),
        i.get_parameter('urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis'),
        i.get_parameter('urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow'),
        i.set_parameter('urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis',["","",1,"",""]),
        i.set_parameter('urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow',["",1,1]),
        i.subscribe("person_detected",""),
        i.subscribe("person_identified",""),
        # i.unsubscribe("1"),
        # i.subscribe("person_detected",""),
        i.get_command_result("sub01-1",""),
        i.disconnect()
        ]
    
    s_e = set()
    for x in range(6):
        e = i.get_event()
        s_e.add((e[0][:3],e[1]))

    r = 0
    if s_e != right_event_result:
        print('event results were wrong')
        print(s_e - right_event_result)
        r=1
    if res != right_command_result:
        print(res)
        r+=10
    q.put(r)
    # print('sa finished')
    return r

class Main_Sub_Engine_test(unittest.TestCase):    
    maxDiff = None
    
    def setUp(self):
        """ setUP
        """

        # start the server process
        self.q = Queue()
        self.ps = [Process(target=test_sa, args=(self.q,)),
            Process(target=HRI_Engine_sample.test_engine, args=(8016,"tests/engine_profile/engine_profile_2_1", "client_info2", None, None,)),
            Process(target=HRI_Engine_sample.test_engine, args=(8017,"tests/engine_profile/engine_profile_2_1", "client_info2", "urn:x-rois:def:HRIEngine:Kyutech::sub01", None,)),
            # Process(target=HRI_Engine_sample.test_engine, args=(8018,"engine_profile_2", "x-rois:def:HRIEngine:Kyutech::SubEng02", "component_client_info", None,)),
            Process(target=Person_Detection.example_pd, args=(8002,)),
            # Process(target=Face_Detection.example_fd, args=(8005,)),
            # Process(target=Face_Localization.example_fl, args=(8006,)),
            # Process(target=Navigation.example_n, args=(8013,)),
            Process(target=Person_Identification.example_pi, args=(8004,)),
            Process(target=Speech_Synthesis.example_ss, args=(8011,)),
            Process(target=Follow.example_f, args=(8014,))
            ]
        
        for x in reversed(self.ps[1:]):
            x.start()
            # print(x.pid)
            time.sleep(0.5)

        time.sleep(5.0)

    def test_main_sub_engine(self):
        """ test_test_main_sub_engine
        """
        self.ps[0].start()
        r = self.q.get(60)
        return self.assertEqual(r,0)

    def tearDown(self):
        """ tearDown
        """
        atexit.register(kill_ps, self.ps)


if __name__ == '__main__':
    # ip = sys.argv
    # if len(ip) > 1:
    #     m = re.search(r'(\d+)\.(\d+)\.(\d+)\.(\d+)',ip[1])
    # else:
    #     m = None
    # if m and len(m.groups()) == 4:
    #     ser_uri = "http://" + ip[1] +":8000"
    #     print("server_url == " + ser_uri + "\n")
    #     del sys.argv[1]
    # else:
    #     ser_uri = "http://127.0.0.1:8000"
    #     print("server_url == " + ser_uri + "\n")
    #     if len(ip) > 1 and ip[1][0] != '-':
    #         del sys.argv[1]

    unittest.main()


# unittest_main_sub_engine4.py
#
# Copyright 2020 Ryota Higashi
# This software may be modified and distributed under the terms
# of the MIT license
#
# for python 3

"""unittest サブエンジンが複数存在する時のテスト
"""

import time
from multiprocessing import Process, Queue
import unittest
import xmlrpc.client
import sys
import re
import logging
import platform
import atexit

from pyrois import RoIS_HRI, RoIS_Common, RoIS_Service
from pyrois_common import HRI_Engine_sample, HRI_Engine_client_sample

from pyrois_common import System_Information
from pyrois_common import Person_Detection
from pyrois_common import Person_Localization
from pyrois_common import Person_Identification
from pyrois_common import Face_Detection
from pyrois_common import Face_Localization
from pyrois_common import Sound_Detection
from pyrois_common import Sound_Localization
from pyrois_common import Speech_Recognition
from pyrois_common import Gesture_Recognition
from pyrois_common import Speech_Synthesis
from pyrois_common import Reaction
from pyrois_common import Navigation
from pyrois_common import Follow
from pyrois_common import Move

eng_profile = """<?xml version="1.0" encoding="UTF-8"?>
<!-->
Robotic Interaction Service Framework Version 1.2
    > Annex A: Examples of Profile in XML
        > A.4 HRI Engine Profile (p.103 (PDF:p.116)) reference
<-->

<rois:HRIEngineProfile gml:id="engine_profile"
    xmlns:rois="http://www.omg.org/spec/RoIS/20151201"
    xmlns:gml="http://www.opengis.net/gml/3.2">
    <gml:identifier>urn:x-rois:def:HRIEngine:Kyutech::main</gml:identifier>
    <gml:name>main</gml:name>
    <rois:SubProfile>
        <gml:identifier>urn:x-rois:def:HRIEngine:Kyutech::sub01</gml:identifier>
        <gml:name>sub01</gml:name>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonDetection</rois:HRIComponent>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonIdentification</rois:HRIComponent>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis</rois:HRIComponent>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow</rois:HRIComponent>
    </rois:SubProfile>
    <rois:SubProfile>
        <gml:identifier>urn:x-rois:def:HRIEngine:Kyutech::sub02</gml:identifier>
        <gml:name>sub02</gml:name>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceDetection</rois:HRIComponent>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceLocalization</rois:HRIComponent>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub02:Navigation</rois:HRIComponent>
    </rois:SubProfile>
    <rois:SubProfile>
        <gml:identifier>urn:x-rois:def:HRIEngine:Kyutech::sub03</gml:identifier>
        <gml:name>sub03</gml:name>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub03:PersonDetection</rois:HRIComponent>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub03:Move</rois:HRIComponent>
    </rois:SubProfile>
</rois:HRIEngineProfile>"""

right_event_result = {
    (('sub01-6', RoIS_Service.Completed_Status.OK), 'completed'), # completed
    (('sub02-5', RoIS_Service.Completed_Status.OK), 'completed'), # completed
    (('sub03-3', RoIS_Service.Completed_Status.OK), 'completed'), # completed
    
    (('sub01-1', 'person_detected', '1'), 'notify_event'),
    (('sub01-2', 'person_detected', '1'), 'notify_event'),
    (('sub01-3', 'person_identified', '2'), 'notify_event'),
    (('sub01-4', 'person_identified', '2'), 'notify_event'),
    (('sub02-1', 'face_localized', '4'), 'notify_event'),
    (('sub02-2', 'face_localized', '4'), 'notify_event'),
    (('sub02-3', 'face_detected', '3'), 'notify_event'),
    (('sub02-4', 'face_detected', '3'), 'notify_event'),
    (('sub03-1', 'person_detected', '1'), 'notify_event'),
    (('sub03-2', 'person_detected', '1'), 'notify_event')
}

right_event_detail = [
    (RoIS_HRI.ReturnCode_t.OK,'person_detected'),
    (RoIS_HRI.ReturnCode_t.OK,'person_detected'),
    (RoIS_HRI.ReturnCode_t.OK,'person_identified'),
    (RoIS_HRI.ReturnCode_t.OK,'person_identified'),
    
    (RoIS_HRI.ReturnCode_t.OK,'face_localized'),
    (RoIS_HRI.ReturnCode_t.OK,'face_localized'),
    (RoIS_HRI.ReturnCode_t.OK,'face_detected'),
    (RoIS_HRI.ReturnCode_t.OK,'face_detected'),
    
    (RoIS_HRI.ReturnCode_t.OK,'person_detected'),
    (RoIS_HRI.ReturnCode_t.OK,'person_detected')
]

right_command_result = [

    RoIS_HRI.ReturnCode_t.OK,
    
    (RoIS_HRI.ReturnCode_t.OK, eng_profile),
    

    (RoIS_HRI.ReturnCode_t.OK,['urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonDetection',
    'urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonIdentification', 
    'urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis', 
    'urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow', 
    'urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceDetection', 
    'urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceLocalization', 
    'urn:x-rois:def:HRIComponent:Kyutech:sub02:Navigation', 
    'urn:x-rois:def:HRIComponent:Kyutech:sub03:PersonDetection', 
    'urn:x-rois:def:HRIComponent:Kyutech:sub03:Move'
    ]),

    RoIS_HRI.ReturnCode_t.OK, # bind:PersonDetection
    RoIS_HRI.ReturnCode_t.OK, # bind:PersonIdentification
    RoIS_HRI.ReturnCode_t.OK, # bind:SpeechSynthesis
    RoIS_HRI.ReturnCode_t.OK, # bind:Follow
    RoIS_HRI.ReturnCode_t.OK, # bind:FaceDetection
    RoIS_HRI.ReturnCode_t.OK, # bind:FaceLocalization
    RoIS_HRI.ReturnCode_t.OK, # bind:Navigation
    RoIS_HRI.ReturnCode_t.OK, # bind:PersonDetection
    RoIS_HRI.ReturnCode_t.OK, # bind:Move

    (RoIS_HRI.ReturnCode_t.OK,"1"), # subscribe:PersonDetection
    (RoIS_HRI.ReturnCode_t.OK,"2"), # subscribe:PersonIdentification
    (RoIS_HRI.ReturnCode_t.OK,"3"), # subscribe:FaceDetection
    (RoIS_HRI.ReturnCode_t.OK,"4"), # subscribe:FaceLocalization
    
    (RoIS_HRI.ReturnCode_t.OK,['', '', 10, '', '', '', '']), # get_parameter:SpeechSynthesis
    (RoIS_HRI.ReturnCode_t.OK,"sub01-6"), # set_parameter:Follow
    (RoIS_HRI.ReturnCode_t.OK,[1000,20]), # get_parameter:FaceLocalization
    (RoIS_HRI.ReturnCode_t.OK,"sub02-5"), # set_parameter:Navigation
    (RoIS_HRI.ReturnCode_t.OK,"sub03-3"), # set_parameter:Move
    
    (RoIS_HRI.ReturnCode_t.OK,['Bind(urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow):Success']), # get_command_result(sub01-4)
    (RoIS_HRI.ReturnCode_t.OK,['Bind(urn:x-rois:def:HRIComponent:Kyutech:sub02:Navigation):Success']), # get_command_result(sub02-3)
    (RoIS_HRI.ReturnCode_t.OK,['Set_parameter(urn:x-rois:def:HRIComponent:Kyutech:sub03:Move)']), # get_command_result(sub03-3)

    ]

def test_sa(q):
    i = HRI_Engine_client_sample.IF("http://127.0.0.1:8016")
    res = [
        i.connect(),
        i.get_profile(""),
        i.search(""),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonDetection'),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonIdentification'),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis'),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow'),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceDetection'),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceLocalization'),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub02:Navigation'),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub03:PersonDetection'),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub03:Move'),
        
        i.subscribe("person_detected",""),
        i.subscribe("person_identified",""),
        i.subscribe("face_detected",""),
        i.subscribe("face_localized",""),
        
        i.get_parameter('urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis'),
        i.set_parameter('urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow',["",1,1]),
        i.get_parameter('urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceLocalization'),
        i.set_parameter('urn:x-rois:def:HRIComponent:Kyutech:sub02:Navigation',[["xx","yy"],100,"aaa"]),
        i.set_parameter('urn:x-rois:def:HRIComponent:Kyutech:sub03:Move',[["line"],["curve"],1]),

        i.get_command_result("sub01-4",""),
        i.get_command_result("sub02-3",""),
        i.get_command_result("sub03-3","")
    ]

    r = 0

    s_e = set()
    for x in range(13):
        e = i.get_event()
        s_e.add((e[0][:3],e[1]))

    # self.assertEqual(s_e,event_result)

    l_ev = []
    ev = [
        i.get_event_detail("sub01-1",""),
        i.get_event_detail("sub01-2",""),
        i.get_event_detail("sub01-3",""),
        i.get_event_detail("sub01-4",""),
        i.get_event_detail("sub02-1",""),
        i.get_event_detail("sub02-2",""),
        i.get_event_detail("sub02-3",""),
        i.get_event_detail("sub02-4",""),
        i.get_event_detail("sub03-1",""),
        i.get_event_detail("sub03-2","")
    ]
    for y in ev:
        l_ev.append((y[0],y[1][0]))
    
    # self.assertEqual(l_ev,event_detail)

    fin = i.disconnect()

    if s_e != right_event_result:
        print('event results were wrong')
        print(s_e - right_event_result)
        print(s_e)
        r=1
    if l_ev != right_event_detail:
        print('event details were wrong')
        print(s_e - right_event_result)
        r+=10
    if res != right_command_result:
        for x, y in zip(res,right_command_result):
            if x != y:
                print('*', x, y)
        r+=100
    q.put(r)
    
    return r

def kill_ps(ps):
    # terminate the server process
    for x in ps:
        x.kill()
        time.sleep(0.5)

class Main_Sub_Engine_test(unittest.TestCase):    
    maxDiff = None
    
    def setUp(self):
        """ setUP
        """

        # start the server process
        self.q = Queue()
        self.ps = [Process(target=test_sa, args=(self.q,)),
            Process(target=HRI_Engine_sample.test_engine, args=(8016,"tests/engine_profile/engine_profile_2_2", "client_info3", None, None,)),
            Process(target=HRI_Engine_sample.test_engine, args=(8017,"tests/engine_profile/engine_profile_2_2", "client_info3", "urn:x-rois:def:HRIEngine:Kyutech::sub01", None,)),
            Process(target=HRI_Engine_sample.test_engine, args=(8018,"tests/engine_profile/engine_profile_2_2", "client_info3", "urn:x-rois:def:HRIEngine:Kyutech::sub02", None,)),
            Process(target=HRI_Engine_sample.test_engine, args=(8019,"tests/engine_profile/engine_profile_2_2", "client_info3", "urn:x-rois:def:HRIEngine:Kyutech::sub03", None,)),
            
            Process(target=Person_Detection.example_pd, args=(8002,)),
            Process(target=Person_Identification.example_pi, args=(8004,)),
            Process(target=Speech_Synthesis.example_ss, args=(8011,)),
            Process(target=Follow.example_f, args=(8014,)),
            
            Process(target=Face_Detection.example_fd, args=(8005,)),
            Process(target=Face_Localization.example_fl, args=(8006,)),
            Process(target=Navigation.example_n, args=(8013,)),

            Process(target=Person_Detection.example_pd, args=(8022,)),
            Process(target=Move.example_m, args=(8035,))
            ]
        # self.ps.reverse()
        
        for x in reversed(self.ps[1:]):
            x.start()
            time.sleep(0.5)

        time.sleep(3.0)
        

    def test_IF(self):
        """ test_IF
        """
        self.ps[0].start()
        r = self.q.get(60)
        return self.assertEqual(r,0)

    def tearDown(self):
        """ tearDown
        """
        atexit.register(kill_ps, self.ps)


if __name__ == '__main__':
    # ip = sys.argv
    # if len(ip) > 1:
    #     m = re.search(r'(\d+)\.(\d+)\.(\d+)\.(\d+)',ip[1])
    # else:
    #     m = None
    # if m and len(m.groups()) == 4:
    #     ser_uri = "http://" + ip[1] +":8000"
    #     print("server_url == " + ser_uri + "\n")
    #     del sys.argv[1]
    # else:
    #     ser_uri = "http://127.0.0.1:8000"
    #     print("server_url == " + ser_uri + "\n")
    #     if len(ip) > 1 and ip[1][0] != '-':
    #         del sys.argv[1]

    unittest.main()


# VirtualNavigation_Client.py
#
# Copyright 2018 Eiichi Inohira
# This software may be modified and distributed under the terms
# of the MIT license
#
# For python3
# For HRI Engine

"""VirtualNavigation_Client
"""

import threading
# import logging
import xmlrpc.client

from pyrois import RoIS_Common, RoIS_HRI


class Command(RoIS_Common.Command):
    """Command
    """
    def __init__(self, uri):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)

    def start(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.start())
        return status

    def stop(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.stop())
        return status

    def suspend(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.suspend())
        return status

    def resume(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.resume())
        return status

    def set_parameter(self, target_position, time_limit, routing_policy):
        status = self._proxy.set_parameter(target_position, time_limit, routing_policy)
        status = RoIS_HRI.ReturnCode_t(status)
        return status
    

class Query(RoIS_Common.Query):
    """Query
    """
    def __init__(self, uri):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)

    def component_status(self):
        status, c_status = self._proxy.component_status()
        return (RoIS_HRI.ReturnCode_t(status), RoIS_Common.Component_Status(c_status))
    
    def get_parameter(self):
        (status, target_position, time_limit, routing_policy) = self._proxy.get_parameter()
        return (RoIS_HRI.ReturnCode_t(status), target_position, time_limit, routing_policy)

import time

class Event(RoIS_Common.Event):
    """Event
    """
    def __init__(self, uri):
        self._uri = uri
        self._e_proxy = xmlrpc.client.ServerProxy(self._uri)
        self.events = []
        # if logger is not None:
        #     self.logger = logger
                
    def start_th(self):
       self.th = threading.Thread(target=self.event_loop)
       self.th.start()

    def event_loop(self):
       """event_loop
       """
       while True:
           time.sleep(1)
           try:
               self.poll_event()
           except xmlrpc.client.ProtocolError as e:
               print("ProtocolError", e)
               continue
           except ConnectionRefusedError as e:
               print("ConnectionRefusedError ", e)
               continue

    def poll_event(self):
       """poll_event
       """
       msg = self._e_proxy.poll_event()
       (params, methodname) = xmlrpc.client.loads(msg)


class VirtualNavigation_Client(Command, Query, Event):
    """VirtualNavigation_Client
    """
    def __init__(self, uri, engine_event_queue):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)
        self._e_proxy = xmlrpc.client.ServerProxy(self._uri)
        self.events = []
        self.engine_event_queue = engine_event_queue
        self.start_th()

    def poll_event(self):
        """poll_event
        """
        msg = self._e_proxy.poll_event()
        (params, methodname) = xmlrpc.client.loads(msg)
        self.engine_event_queue.put(msg)



import xml.etree.ElementTree as ET
import message_profile
tree = ET.parse('component_profile/component_profile.xml')
root = tree.getroot()
# d = {}
print(root.tag, root.attrib)
if 'HRIComponentProfile' in root.tag:
    p = message_profile.HRI_Component_Profile()
    for x in root:
        # d[x.tag] = x.text
        print(x.tag, x.attrib)
        if 'identifier' in x.tag:
            p.identifier = x.text.lstrip()
        if 'name' in x.tag:
            p.name = x.text.lstrip()
        if 'MessageProfile' in x.tag:
            m = message_profile.Message_Profile()
            m.name = x.attrib["{http://www.omg.org/spec/RoIS/20151201}name"]
            m.type = x.attrib["{http://www.w3.org/2001/XMLSchema-instance}type"]
            p.add_message(m)
            for y in x:
                print('y', y.tag, y.attrib)
                if 'Results' in y.tag:
                    r = message_profile.Parameter_Profile()
                    r.name = y.attrib["{http://www.omg.org/spec/RoIS/20151201}name"]
                    m.add_result(r)
                    for z in y:
                        print(z.tag, z.attrib)
                        if 'data_type_ref' in z.tag:
                            r.data_type_ref = z.attrib["{http://www.omg.org/spec/RoIS/20151201}code"]


p.show()

# e = message_profile.HRI_Engine_Profile(d["{http://www.opengis.net/gml/3.2}identifier"], d["{http://www.opengis.net/gml/3.2}name"])
# print(e)


# Face_Detection_Client.py
#
# Copyright 2019 Ryota Higashi

# This software may be modified and distributed under the terms
# of the MIT license
#
# For python3
# For HRI Engine

"""Face_Detection_Client
"""

import threading
# import logging
import xmlrpc.client
from queue import Queue

from pyrois import RoIS_Common, RoIS_HRI


class Command(RoIS_Common.Command):
    """Command
    """
    def __init__(self, uri):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)

    def start(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.start())
        return status

    def stop(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.stop())
        return status

    def suspend(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.suspend())
        return status

    def resume(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.resume())
        return status


class Query(RoIS_Common.Query):
    """Query
    """
    def __init__(self, uri):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)

    def component_status(self):
        (status, c_status) = self._proxy.component_status()
        return (RoIS_HRI.ReturnCode_t(status), RoIS_Common.Component_Status(c_status))


class Event(RoIS_Common.Event):
    """Event
    """
    def __init__(self, uri):
        self._uri = uri
        self._e_proxy = xmlrpc.client.ServerProxy(self._uri)
        self.events = []
        # if logger is not None:
        #     self.logger = logger

    def start_th(self):
        self.th = threading.Thread(target=self.event_loop, daemon=True)
        self.th.start()

    def event_loop(self):
        """event_loop
        """
        while True:
            try:
                self.poll_event()
            except ConnectionRefusedError:
                break

    def poll_event(self):
        """poll_event
        """
        msg = self._e_proxy.poll_event()
        (params, methodname) = xmlrpc.client.loads(msg)
        #self.logger.debug('poll_event: '+methodname)
        # print(params,methodname)
        if methodname == 'face_detected':
            self.face_detected(params[0], params[1])

    def face_detected(self, timestamp, number):
        self.events.append((timestamp,number))
        if self.q is not None:
            self.q.put(("face_detected",timestamp,number))
        # print("face_detected",timestamp,number)
        # self.logger.debug('received completed event'
        #                 + command_id
        #                 + RoIS_Service.Completed_Status(status).name)


class Face_Detection_Client(Command, Query, Event):
    """Face_Detection_Client
    """
    def __init__(self, uri, q=None):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)
        self._e_proxy = xmlrpc.client.ServerProxy(self._uri)
        self.events = []
        self.q = q
        self.start_th()



# command_message.py
#
# Copyright 2019 Ryota Higashi

from typing import List




class MD_Identifier:  # [ISO19115]
    def __init__(self, authority: str, code: str):
        self.authority = authority  # https://www.wmo.int/schemas/wmdr/1.0RC8/documentation/schemadoc/schemas/referenceSystem_xsd/elements/authority.html
        self.code = code            # https://www.wmo.int/schemas/wmdr/1.0RC4/documentation/schemadoc/schemas/identification_xsd/elements/MD_DataIdentification.html

class RoIS_Identifier(MD_Identifier):
    def __init__(self, codebook_reference: str, version: str):
        self.codebook_reference = codebook_reference
        self.version = version



class Parameter:
    def __init__(self, name: str, data_type_ref: RoIS_Identifier, value: any):
        self.name = name                    # use="required"
        self.data_type_ref = data_type_ref  # minOccurs="1" maxOccurs="1"
        self.value = value                  # minOccurs="1" maxOccurs="1"

class ResultList:
    def __init__(self, parameters: List[Parameter]):
        self.parameters = parameters

class ArgumentList:
    def __init__(self, parameters: List[Parameter]):
        self.parameters = parameters       # minOccurs="1" maxOccurs="unbounded"

class ParameterList:
    def __init__(self, parameters: List[Parameter]):
        self.parameters = parameters

class Command_Result_Message:
    def __init__(self, command_id: str, condition: str, results: ResultList):
        self.command_id = command_id
        self.condition = condition      # QueryExpression[iso19143] = filterCondition(xml)
        self.results = results

class Query_Message:
    def __init__(self, query_type: str, condition: str, results: ResultList):
        self.query_type = query_type
        self.condition = condition
        self.results = results

class Event_Message:
    def __init__(self, event_id: str, event_type: str, subscribe_id: str, expire: str): #expire: "YYYY-MM-DDTHH:MM:SS"
        self.event_id = event_id
        self.event_type = event_type
        self.subscribe_id = subscribe_id
        self.expire = expire        

class Event_Detail_Message:
    def __init__(self, event_id: str, condition: str, results: ResultList):
        self.event_id = event_id
        self.condition = condition
        self.results = results

class Error_Message:
    def __init__(self, error_id: str, error_type: str, subscribe_id: str, expire: str):
        self.error_id = error_id
        self.error_type = error_type
        self.subscribe_id = subscribe_id
        self.expire = expire

class Error_Detail_Message:
    def __init__(self, error_id: str, condition: str, results: ResultList):
        self.error_id = error_id
        self.condition = condition
        self.results = results

###

class CommandBase:
    def __init__(self, delay_time: int):        # delay_time: int
        self.delay_time = delay_time            # use="optional"

class CommandUnitSequence:
    def __init__(self, l: List[CommandBase]): 
        self.command_unit_list = l              # minOccurs="1" maxOccurs="unbounded"

class CommandMessage(CommandBase):
    def __init__(self, component_ref: RoIS_Identifier, command_type: str, command_id: str, arguments: ArgumentList):
        self.component_ref = component_ref      # minOccurs="1" maxOccurs="1"
        self.command_type = command_type        # use="required"
        self.command_id = command_id            # use="required"
        self.arguments = arguments              # minOccurs="0" maxOccurs="1"
    
class Branch:
    def __init__(self, command_list: List[CommandMessage]):
        self.command_list = command_list        # maxOccurs=”unbounded”

class ConcurrentCommands(CommandBase):
    def __init__(self, branch_list: List[Branch]):
        self.branch_list = branch_list


# unittest.py
#
# Copyright 2018 Eiichi Inohira
# This software may be modified and distributed under the terms
# of the MIT license
#
# for python 3

"""unittest
"""

import time
import threading
from multiprocessing import Process, Queue
import unittest
import http
import xmlrpc.client
import logging
import logging.handlers
from datetime import datetime

from pyrois import RoIS_HRI, RoIS_Common, RoIS_Service
from pyrois import Service_Application_IF, Service_Application_Base_sample
from pyrois import HRI_Engine_client, HRI_Engine_sample
from pyrois import Person_Detection_client, Person_Detection
from pyrois import IF_server

def test_engine(port):
    """test_engine
    """
    IF_server.IF_server(port).run(HRI_Engine_sample.IF(HRI_Engine_sample.MyHRIE()))

def event_dispatch(sa):
    """event_dispatch
    """
    sa.completed("0", RoIS_Service.Completed_Status.OK.value)
    time.sleep(0.1)
    sa.notify_error("1", RoIS_Service.ErrorType.ENGINE_INTERNAL_ERROR.value)
    time.sleep(0.1)
    sa.notify_event("2", "sensor", "0", "2100-01-01T00:00:01+09:00")

# def event_dispatch_long(sa):
#     sa.completed("0", RoIS_Service.Completed_Status.OK.value)
#     time.sleep(1)
#     sa.notify_error("0", RoIS_Service.ErrorType.ENGINE_INTERNAL_ERROR.value)
#     time.sleep(60)
#     sa.notify_event("0", "sensor", "0", "2100-01-01T00:00:01+09:00")

def sample_sa(port):
    """sample_sa
    """
    sa = Service_Application_Base_sample.Service_Application_Base()

    # start the timer to dispatch events
    t = threading.Timer(0.1, event_dispatch, args=(sa,))
    t.start()

    # start the XML-RPC server
    IF_server.IF_server(port).run(sa)

def sample_sa_IF(url, q):
    try:
        logger = logging.getLogger('Service_Application_IF')
        logger.setLevel(logging.DEBUG)
        ch = logging.handlers.QueueHandler(q)
        # ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)
        formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        logger.addHandler(ch)
        a = Service_Application_IF.Service_Application_IF(url, logger=logger)
        while True:
            time.sleep(5)
        print("Finished")
    except KeyboardInterrupt:
        print("Interrupted")

def pd_event_dispatch(pd):
    """event_dispatch
    """
    pd.person_detected(datetime.now().isoformat(), 1)
    time.sleep(0.1)
    pd.person_detected(datetime.now().isoformat(), 1)


def sample_pd(port):
    """sample_pd
    """
    c = Person_Detection.component()
    pd = Person_Detection.Person_Detection(c)

    # start the timer to dispatch events
    t = threading.Timer(0.1, pd_event_dispatch, args=(pd,))
    t.start()

    # start the XML-RPC server
    IF_server.IF_server(port).run(pd)

class TestServericeApplicationIF(unittest.TestCase):
    """TestServericeApplicationIF
    """
    def setUp(self):
        # start the server process
        self.ps = Process(
            target=sample_sa, args=(8000,))
        self.ps.start()
        time.sleep(0.1)

    def test_IF(self):
        """ test_IF
        """

        q = Queue()
        # start the client process
        pc = Process(target=sample_sa_IF,
                     args=("http://127.0.0.1:8000/", q))
        pc.start()

        time.sleep(1)

        num_events = 0
        while not q.empty():
            msg = q.get()
            num_events += 1
            # print(msg.asctime,msg.name,msg.levelname,msg.message)

        if pc.is_alive():
            pc.terminate()

        return self.assertEqual(num_events, 3)

    def tearDown(self):
        # terminate the server process
        if self.ps.is_alive():
            self.ps.terminate()
            while self.ps.is_alive():
                time.sleep(0.1)


class TestHRIEngineIF(unittest.TestCase):
    """TestHRIEngineIF
    """

    def setUp(self):
        # start the server process
        self.ps = Process(target=test_engine, args=(8000,))
        self.ps.start()
        time.sleep(0.1)

    def test_IF(self):
        """test_IF
        """

        with xmlrpc.client.ServerProxy("http://127.0.0.1:8000/") as proxy:
            a = HRI_Engine_client.SystemIF(proxy)
            b = HRI_Engine_client.CommandIF(proxy)

            try:
                res = [
                    a.connect(),
                    a.connect(),
                    b.bind(''),
                    a.disconnect(),
                    a.disconnect(),
                    b.bind('')]
                # print(res)
            except xmlrpc.client.ProtocolError as err:
                print(err)

        return self.assertEqual(res,
                                [
                                    RoIS_HRI.ReturnCode_t.OK,
                                    RoIS_HRI.ReturnCode_t.ERROR,
                                    RoIS_HRI.ReturnCode_t.OK,
                                    RoIS_HRI.ReturnCode_t.OK,
                                    RoIS_HRI.ReturnCode_t.ERROR,
                                    RoIS_HRI.ReturnCode_t.OK])

    def tearDown(self):
        # terminate the server process
        if self.ps.is_alive():
            self.ps.terminate()
            time.sleep(0.1)


class TestHRIEngineIF_integrated(unittest.TestCase):
    """TestHRIEngineIF_integrated
    """
    def setUp(self):
        # start the server process
        self.ps = Process(target=test_engine, args=(8000,))
        self.ps.start()
        time.sleep(0.1)

    def test_IF(self):
        """test_IF
        """

        a = HRI_Engine_client.IF("http://127.0.0.1:8000/")
            

        try:
            res = [
                a.connect(),
                a.connect(),
                a.get_profile(""),
                a.get_error_detail("0", ""),

                a.search(""),
                a.bind(""),
                a.bind_any(""),
                a.release(""),
                a.get_parameter(""),
                a.set_parameter("", ""),
                a.execute(""),
                a.get_command_result("", ""),

                a.query("", ""),

                a.subscribe("", ""),
                a.unsubscribe(""),
                a.get_event_detail("", ""),

                a.disconnect(),
                a.disconnect(),
                a.bind('')]
                # print(res)
        except xmlrpc.client.ProtocolError as err:
            print(err)

        return self.assertEqual(res,
                                [RoIS_HRI.ReturnCode_t.OK,
                                 RoIS_HRI.ReturnCode_t.ERROR,
                                 (RoIS_HRI.ReturnCode_t.OK, "Unsupported"),
                                 (RoIS_HRI.ReturnCode_t.OK, "None"),

                                 (RoIS_HRI.ReturnCode_t.OK, ["None"]),
                                 RoIS_HRI.ReturnCode_t.OK,
                                 (RoIS_HRI.ReturnCode_t.OK, ["None"]),
                                 RoIS_HRI.ReturnCode_t.OK,
                                 (RoIS_HRI.ReturnCode_t.OK, ["None"]),
                                 (RoIS_HRI.ReturnCode_t.OK, "0"),
                                 RoIS_HRI.ReturnCode_t.OK,
                                 (RoIS_HRI.ReturnCode_t.OK, ["None"]),

                                 (RoIS_HRI.ReturnCode_t.OK, ["None"]),

                                 (RoIS_HRI.ReturnCode_t.OK, "0"),
                                 RoIS_HRI.ReturnCode_t.OK,
                                 (RoIS_HRI.ReturnCode_t.OK, ["None"]),

                                 RoIS_HRI.ReturnCode_t.OK,
                                 RoIS_HRI.ReturnCode_t.ERROR,
                                 RoIS_HRI.ReturnCode_t.OK])

    def tearDown(self):
        # terminate the server process
        if self.ps.is_alive():
            self.ps.terminate()
            time.sleep(0.1)


class TestPD(unittest.TestCase):
    """TestPD
    """

    def setUp(self):
        # start the server process
        self.ps = Process(target=sample_pd, args=(8000,))
        self.ps.start()
        time.sleep(1)

    def test_IF(self):
        """test_IF
        """

        a = Person_Detection_client.Person_Detection_Client("http://127.0.0.1:8000/")

        try:
            res = [
                a.start(),
                a.stop(),
                a.suspend(),
                a.resume(),
                a.component_status()]
            # print(res)
        except xmlrpc.client.ProtocolError as err:
            print(err)
        except http.client.CannotSendRequest as err:
            print(err)

        time.sleep(1)

        return self.assertEqual(res,
                                [
                                    RoIS_HRI.ReturnCode_t.OK,
                                    RoIS_HRI.ReturnCode_t.OK,
                                    RoIS_HRI.ReturnCode_t.OK,
                                    RoIS_HRI.ReturnCode_t.OK,
                                    (RoIS_HRI.ReturnCode_t.OK, RoIS_Common.Component_Status.UNINITIALIZED)])

    def tearDown(self):
        # terminate the server process
        if self.ps.is_alive():
            self.ps.terminate()
            time.sleep(0.1)


if __name__ == '__main__':
    unittest.main()



# Speech_Synthesis.py
#
# Copyright 2019 Ryota Higashi
# This software may be modified and distributed under the terms
# of the MIT license
#
# For python3
# For HRI Component

"""Speech_Synthesis
"""

import sys
import queue
import time
from datetime import datetime
import threading
import logging

from pyrois import RoIS_Common, RoIS_HRI
from . import Component_Sample

import socketserver
import xmlrpc.server

class ThreadingXMLRPCServer(socketserver.ThreadingMixIn, xmlrpc.server.SimpleXMLRPCServer):
    """ThreadingXMLRPCServer
    """
    pass


class Command(RoIS_Common.Command):
    """Command
    """
    def __init__(self, c):
        self._component = c
        
    def start(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        self.c_status = RoIS_Common.Component_Status.BUSY.value
        return status

    def stop(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        self.c_status = RoIS_Common.Component_Status.UNINITIALIZED.value
        return status

    def suspend(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        self.c_status = RoIS_Common.Component_Status.READY.value
        return status

    def resume(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        self.c_status = RoIS_Common.Component_Status.BUSY.value
        return status
    
    def set_parameter(self, speech_text, SSML_text, volume, languages, character):
        arg = [speech_text, SSML_text, volume, languages, character]
        self.parameter_queue.put(arg)
        self.Speech_Text = speech_text
        self.SSML_Text = SSML_text
        self.Volume = volume
        self.Languages = languages
        self.Character = character
        # self.logger.info("Speech_Synthesis set_parameter")
        
        th = threading.Thread(target=self.execute, daemon=True)
        th.start()
        
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status


class Query(RoIS_Common.Query):
    """Query
    """
    def __init__(self, c):
        self._component = c

    def component_status(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        with self._lock:
            c_status = self.c_status
        #print(c_status)
        # self.logger.info("component_satus:{}".format(c_status))
        return (status, c_status)

    def get_parameter(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        self.c_status = RoIS_Common.Component_Status.READY.value
        return (status, self.Speech_Text, self.SSML_Text,
            self.Volume, self.Languages, self.Character,
            self.Synthesizable_Languages, self.Synthesizable_Characters)
    

class Event(RoIS_Common.Event):
    """Event
    """
    def __init__(self, c):
        self._component = c
        self.event_queue = queue.Queue()

    # def poll_event(self):
    #     """poll_event
    #     """
    #     msg = self.event_queue.get()
    #     return msg


class Speech_Synthesis(Event, Command, Query):
    """Speech_Synthesis
    """
    def __init__(self, c, lock=None):
        if lock is None:
            self._lock = threading.Lock()
        else:
            self._lock = lock
        
        self._component = c
        self.c_status = RoIS_Common.Component_Status.READY.value
        
        self.Speech_Text = ""
        self.SSML_Text = ""
        self.Volume = 10
        self.Languages = ""
        self.Character = ""
        self.Synthesizable_Languages = ("")
        self.Synthesizable_Characters = ("")
        self.parameter_queue = queue.Queue()
        self.comp = Component_Sample.Component_Sample()

        # self.logger = logging.getLogger('Speech_Synthesis_server')
        # self.logger.setLevel(logging.DEBUG)
        # formatter = logging.Formatter('%(asctime)s - %(name)s - %(message)s')
        # self.fh = logging.FileHandler('Speech_Synthesis(many_components_test).log')    # 場合によって変える
        # self.fh.setLevel(logging.DEBUG)
        # self.fh.setFormatter(formatter)
        # self.logger.addHandler(self.fh)

        # self.ch = logging.StreamHandler()
        # self.ch.setLevel(logging.INFO)
        # self.ch.setFormatter(formatter)
        # self.logger.addHandler(self.ch)

        # self.logger.debug("\n\n*****************\n")
        # self.logger.debug("Speech_Synthesis_Server running")

    def execute(self):
        #while True:
        arg = self.parameter_queue.get()
        with self._lock:
            self.c_status = RoIS_Common.Component_Status.BUSY.value
    
        # y = self.comp.Speech_Synthesis(*arg)
        # with self._lock:
        #    self.c_status = y

        # self.logger.info("Speech_Synthesis speech_now")
        time.sleep(1)
        with self._lock:
            self.c_status = RoIS_Common.Component_Status.READY.value


class component:
    """component
    """
    def __init__(self):
        self._state = False


def event_dispatch(ss):
    """event_dispatch
    """


def example_ss(port):
    """example_ss
    """
    c = component()
    lock = threading.Lock()
    ss = Speech_Synthesis(c,lock)

    # start the timer to dispatch events
    t = threading.Timer(0.1, event_dispatch, args=(ss,))
    t.start()

    # start the XML-RPC server
    server = ThreadingXMLRPCServer(("0.0.0.0", port), logRequests=False)
    server.register_instance(ss)
    server.register_introspection_functions()
    server.register_multicall_functions()
    #print("Speech_Synthesis running")
    server.serve_forever()


if __name__ == '__main__':
    example_ss(8011)


import xml.etree.ElementTree as ET
import message_profile_eng
tree = ET.parse('engine_profile/engine_profile_2.xml')
root = tree.getroot()
# d = {}
print(root.tag, root.attrib, '\n')
if 'HRIEngineProfile' in root.tag:
    e = message_profile_eng.HRI_Engine_Profile()
    for x in root:
        # d[x.tag] = x.text
        #print(x.tag, x.attrib)
        if 'identifier' in x.tag:
            e.identifier = x.text.lstrip()  #lstrip:左端の文字列を削除
                                            # ただ引数に何もない。必要性は？
        if 'name' in x.tag:
            e.name = x.text.lstrip()
        if 'SubProfile' in x.tag:
            #e.sub_profile = x.text.lstrip()
            s = message_profile_eng.HRI_Engine_Profile()
            for y in x:
                #print('y', y.tag, y.attrib)
                if 'identifier' in y.tag:
                    #print(y.text)
                    s.identifier = y.text.lstrip()
                    #e.add_engine(y.text)
                    #e.sub_profile = y.text.lstrip()
                if 'name' in y.tag:
                    #print(y.text)
                    s.name = y.text.lstrip()
                    e.add_engine(s)
                    #e.add_engine(y.text)
                    #e.sub_profile = y.text.lstrip()
                if 'HRIComponent' in y.tag:
                    c = message_profile_eng.HRI_Component_Profile()
                    c.identifier = y.text.lstrip() #これをprintすればコンポーネントのIDが出力できる
                    s.add_component(c) 
                    #e.component = y.text.lstrip()
                    #e.add_component(y.text)
                    #print(y.text.lstrip())
        elif 'HRIComponent' in x.tag:
            c = message_profile_eng.HRI_Component_Profile()
            c.identifier = x.text.lstrip()
            e.add_component(c)
                
e.show()

# e = message_profile.HRI_Engine_Profile(d["{http://www.opengis.net/gml/3.2}identifier"], d["{http://www.opengis.net/gml/3.2}name"])
# print(e)


# Move.py
#
# Copyright 2019 Ryota Higashi
# This software may be modified and distributed under the terms
# of the MIT license
#
# For python3
# For HRI Component

"""Move
"""

import sys
import queue
import time
from datetime import datetime
import threading

from pyrois import RoIS_Common, RoIS_HRI

import socketserver
import xmlrpc.server

class ThreadingXMLRPCServer(socketserver.ThreadingMixIn, xmlrpc.server.SimpleXMLRPCServer):
    """ThreadingXMLRPCServer
    """
    pass


class Command(RoIS_Common.Command):
    """Command
    """
    def __init__(self, c):
        self._component = c

    def start(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def stop(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def suspend(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def resume(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def set_parameter(self, line, curve, time):
        arg = [line, curve, time]
        self.parameter_queue.put(arg)
        self.Line = line
        self.Curve = curve
        self.Time = time

        self.th = threading.Thread(target=self.execute, daemon=True)
        self.th.start()
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status


class Query(RoIS_Common.Query):
    """Query
    """
    def __init__(self, c):
        self._component = c

    def component_status(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        with self._lock:
            c_status = self.c_status
        return (status, c_status)
    
    def get_parameter(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        self.c_status = RoIS_Common.Component_Status.READY.value
        return (status, self.Line, self.Curve, self.Time)
    

class Event(RoIS_Common.Event):
    """Event
    """
    def __init__(self, c):
        self._component = c
        self.event_queue = queue.Queue()

    # def poll_event(self):
    #     """poll_event
    #     """
    #     msg = self.event_queue.get()
    #     return msg


class Move(Event, Command, Query):
    """Move
    """
    def __init__(self, c, lock=None):
        super().__init__(c)
        self._component = c
        if lock is None:
            self._lock = threading.Lock()
        else:
            self._lock = lock
        self.c_status = RoIS_Common.Component_Status.READY.value
        self.parameter_queue = queue.Queue()

        self.Line = [""]
        self.Curve = [""]
        self.Time = 10
    
    def execute(self):  # dummy
        """execute component
        """
        arg = self.parameter_queue.get()
        with self._lock:
            self.c_status = RoIS_Common.Component_Status.BUSY.value

        time.sleep(1)

        with self._lock:
            self.c_status = RoIS_Common.Component_Status.READY.value


class component:
    """component
    """
    def __init__(self):
        self._state = False


def event_dispatch(m):
    """event_dispatch
    """


def example_m(port):
    """example_m
    """
    c = component()
    m = Move(c)
    # start the timer to dispatch events
    t = threading.Timer(0.1, event_dispatch, args=(m,))
    t.start()

    # start the XML-RPC server
    server = ThreadingXMLRPCServer(("0.0.0.0", port), logRequests=False)
    server.register_instance(m)
    server.register_introspection_functions()
    server.register_multicall_functions()
    server.serve_forever()


if __name__ == '__main__':
    example_m(8015)


# unittest_main_sub_engine3.py
#
# Copyright 2020 Ryota Higashi
# This software may be modified and distributed under the terms
# of the MIT license
#
# for python 3

"""unittest サブエンジンが存在しない場合のテスト
"""

import time
from multiprocessing import Process, Queue
import unittest
import xmlrpc.client
import sys
import re
import logging
import platform
from datetime import datetime
import json

from pyrois import RoIS_HRI, RoIS_Common, RoIS_Service
import HRI_Engine_sample, HRI_Engine_client_sample
import command_message

from component import System_Information
from component import Person_Detection
from component import Person_Localization
from component import Person_Identification
from component import Face_Detection
from component import Face_Localization
from component import Sound_Detection
from component import Sound_Localization
from component import Speech_Recognition
from component import Gesture_Recognition
from component import Speech_Synthesis
from component import Reaction
from component import Navigation
from component import Follow
from component import Move


eng_profile = """<?xml version="1.0" encoding="UTF-8"?>
<!-->
Robotic Interaction Service Framework Version 1.2
    > Annex A: Examples of Profile in XML
        > A.4 HRI Engine Profile (p.103 (PDF:p.116)) reference
<-->

<rois:HRIEngineProfile gml:id="engine_profile"
    xmlns:rois="http://www.omg.org/spec/RoIS/20151201"
    xmlns:gml="http://www.opengis.net/gml/3.2">
    <gml:identifier>urn:x-rois:def:HRIEngine:Kyutech::main</gml:identifier>
    <gml:name>main</gml:name>
    <rois:SubProfile>
        <gml:identifier>urn:x-rois:def:HRIEngine:Kyutech::sub01</gml:identifier>
        <gml:name>sub01</gml:name>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonDetection</rois:HRIComponent>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonIdentification</rois:HRIComponent>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis</rois:HRIComponent>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow</rois:HRIComponent>
    </rois:SubProfile>
    <rois:SubProfile>
        <gml:identifier>urn:x-rois:def:HRIEngine:Kyutech::sub02</gml:identifier>
        <gml:name>sub02</gml:name>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceDetection</rois:HRIComponent>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceLocalization</rois:HRIComponent>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub02:Navigation</rois:HRIComponent>
    </rois:SubProfile>
    <rois:SubProfile>
        <gml:identifier>urn:x-rois:def:HRIEngine:Kyutech::sub03</gml:identifier>
        <gml:name>sub03</gml:name>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub03:PersonDetection</rois:HRIComponent>
        <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:sub03:Move</rois:HRIComponent>
    </rois:SubProfile>
</rois:HRIEngineProfile>"""


command_unit_list1 = """{
    "command_unit_list": [
        {
            "component_ref": "urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis",
            "command_type": "set_parameter",
            "command_id": "sub01-5",
            "arguments": {
                "parameters": [
                    {
                        "name": "speech_text",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": "aaa"
                    },
                    {
                        "name": "SSML_text",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": "ssml"
                    },
                    {
                        "name": "volume",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": 50
                    },
                    {
                        "name": "languages",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": "japanese"
                    },
                    {
                        "name": "character",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": "m1"
                    }
                ]
            },
            "delay_time": 0
        },
        {
            "component_ref": "urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis",
            "command_type": "get_parameter",
            "command_id": "main-exe-1-2",
            "arguments": {
                "parameters": [
                    {
                        "name": "",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": ""
                    }
                ]
            },
            "delay_time": 0
        },
        {
            "branch_list": [
                {
                    "command_list": [
                        {
                            "component_ref": "urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceLocalization",
                            "command_type": "set_parameter",
                            "command_id": "sub02-4",
                            "arguments": {
                                "parameters": [
                                    {
                                        "name": "detection_threshold",
                                        "data_type_ref": {
                                            "codebook_reference": "",
                                            "version": ""
                                        },
                                        "value": 3
                                    },
                                    {
                                        "name": "minimum_interval",
                                        "data_type_ref": {
                                            "codebook_reference": "",
                                            "version": ""
                                        },
                                        "value": 2
                                    }
                                ]
                            },
                            "delay_time": 3000
                        },
                        {
                            "component_ref": "urn:x-rois:def:HRIComponent:Kyutech:sub02:Navigation",
                            "command_type": "set_parameter",
                            "command_id": "sub02-5",
                            "arguments": {
                                "parameters": [
                                    {
                                        "name": "target_position",
                                        "data_type_ref": {
                                            "codebook_reference": "",
                                            "version": ""
                                        },
                                        "value": [
                                            100.0,
                                            200.0
                                        ]
                                    },
                                    {
                                        "name": "time_limit",
                                        "data_type_ref": {
                                            "codebook_reference": "",
                                            "version": ""
                                        },
                                        "value": 60000
                                    },
                                    {
                                        "name": "routing_policy",
                                        "data_type_ref": {
                                            "codebook_reference": "",
                                            "version": ""
                                        },
                                        "value": "time priority"
                                    }
                                ]
                            },
                            "delay_time": 4000
                        }
                    ]
                },
                {
                    "command_list": [
                        {
                            "component_ref": "urn:x-rois:def:HRIComponent:Kyutech:sub03:Move",
                            "command_type": "set_parameter",
                            "command_id": "sub03-3",
                            "arguments": {
                                "parameters": [
                                    {
                                        "name": "line",
                                        "data_type_ref": {
                                            "codebook_reference": "",
                                            "version": ""
                                        },
                                        "value": [
                                            1000,
                                            360
                                        ]
                                    },
                                    {
                                        "name": "curve",
                                        "data_type_ref": {
                                            "codebook_reference": "",
                                            "version": ""
                                        },
                                        "value": [
                                            1000,
                                            90
                                        ]
                                    },
                                    {
                                        "name": "time",
                                        "data_type_ref": {
                                            "codebook_reference": "",
                                            "version": ""
                                        },
                                        "value": 1000
                                    }
                                ]
                            },
                            "delay_time": 5000
                        }
                    ]
                }
            ],
            "delay_time": 0
        },
        {
            "component_ref": "urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow",
            "command_type": "set_parameter",
            "command_id": "sub01-7",
            "arguments": {
                "parameters": [
                    {
                        "name": "target_object_ref",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": "human1"
                    },
                    {
                        "name": "distance",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": 500
                    },
                    {
                        "name": "time_limit",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": 60000
                    }
                ]
            },
            "delay_time": 1000
        }
    ]
}"""

# command_unit_list_dict1 = {'command_unit_list': [{'component_ref': 'urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis', 'command_type': 'set_parameter', 'command_id': 'sub01-5', 'arguments': {'parameters': [{'name': 'speech_text', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 'aaa'}, {'name': 'SSML_text', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 'ssml'}, {'name': 'volume', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 50}, {'name': 'languages', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 'japanese'}, {'name': 'character', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 'm1'}]}, 'delay_time': 0}, {'component_ref': 'urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis', 'command_type': 'get_parameter', 'command_id': 'main-exe-1-2', 'arguments': {'parameters': [{'name': '', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': ''}]}, 'delay_time': 0}, {'branch_list': [{'command_list': [{'component_ref': 'urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceLocalization', 'command_type': 'set_parameter', 'command_id': 'sub02-4', 'arguments': {'parameters': [{'name': 'detection_threshold', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 3}, {'name': 'minimum_interval', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 2}]}, 'delay_time': 3000}, {'component_ref': 'urn:x-rois:def:HRIComponent:Kyutech:sub02:Navigation', 'command_type': 'set_parameter', 'command_id': 'sub02-5', 'arguments': {'parameters': [{'name': 'target_position', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': [100.0, 200.0]}, {'name': 'time_limit', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 60000}, {'name': 'routing_policy', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 'time priority'}]}, 'delay_time': 4000}]}, {'command_list': [{'component_ref': 'urn:x-rois:def:HRIComponent:Kyutech:sub03:Move', 'command_type': 'set_parameter', 'command_id': 'sub03-3', 'arguments': {'parameters': [{'name': 'line', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': [1000, 360]}, {'name': 'curve', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': [1000, 90]}, {'name': 'time', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 1000}]}, 'delay_time': 5000}]}], 'delay_time': 0}, {'component_ref': 'urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow', 'command_type': 'set_parameter', 'command_id': 'sub01-7', 'arguments': {'parameters': [{'name': 'target_object_ref', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 'human1'}, {'name': 'distance', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 500}, {'name': 'time_limit', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 60000}]}, 'delay_time': 1000}]}

event_result = {
    (('sub01-5', RoIS_Service.Completed_Status.OK), 'completed'), # completed
    (('sub02-4', RoIS_Service.Completed_Status.OK), 'completed'), # completed
    (('sub02-5', RoIS_Service.Completed_Status.OK), 'completed'), # completed
    (('sub03-3', RoIS_Service.Completed_Status.OK), 'completed'), # completed
    (('sub01-7', RoIS_Service.Completed_Status.OK), 'completed'), # completed
    (('sub01-1', 'person_detected', '1'), 'notify_event'),
    (('sub01-2', 'person_detected', '1'), 'notify_event'),
    (('sub01-3', 'person_identified', '2'), 'notify_event'),
    (('sub01-4', 'person_identified', '2'), 'notify_event'),
    (('sub02-1', 'face_localized', '4'), 'notify_event'),
    (('sub02-2', 'face_localized', '4'), 'notify_event'),
    (('sub02-3', 'face_detected', '3'), 'notify_event'),
    (('sub02-4', 'face_detected', '3'), 'notify_event'),
    (('sub03-1', 'person_detected', '1'), 'notify_event'),
    (('sub03-2', 'person_detected', '1'), 'notify_event')
}


class Execute_test(unittest.TestCase):    
    maxDiff = None
    
    def setUp(self):
        """ setUP
        """
        # start the server process
        self.ps = [Process(target=HRI_Engine_sample.test_engine, args=(8016,"engine_profile_2_2", "client_info3", None, None,)),
            Process(target=HRI_Engine_sample.test_engine, args=(8017,"engine_profile_2_2", "client_info3", "urn:x-rois:def:HRIEngine:Kyutech::sub01", None,)),
            Process(target=HRI_Engine_sample.test_engine, args=(8018,"engine_profile_2_2", "client_info3", "urn:x-rois:def:HRIEngine:Kyutech::sub02", None,)),
            Process(target=HRI_Engine_sample.test_engine, args=(8019,"engine_profile_2_2", "client_info3", "urn:x-rois:def:HRIEngine:Kyutech::sub03", None,)),
            
            Process(target=Person_Detection.example_pd, args=(8002,)),
            Process(target=Person_Identification.example_pi, args=(8004,)),
            Process(target=Speech_Synthesis.example_ss, args=(8011,)),
            Process(target=Follow.example_f, args=(8014,)),
            
            Process(target=Face_Detection.example_fd, args=(8005,)),
            Process(target=Face_Localization.example_fl, args=(8006,)),
            Process(target=Navigation.example_n, args=(8013,)),

            Process(target=Person_Detection.example_pd, args=(8022,)),
            Process(target=Move.example_m, args=(8035,))
            ]
        
        for x in reversed(self.ps):
            x.start()
            time.sleep(0.5)

        time.sleep(5.0)
        

    def test_IF(self):
        """ test_IF
        """
        #execute1
        c_ri = command_message.RoIS_Identifier("","")
        c_p_ss1 = command_message.Parameter("speech_text", c_ri, "aaa")
        c_p_ss2 = command_message.Parameter("SSML_text", c_ri, "ssml")
        c_p_ss3 = command_message.Parameter("volume", c_ri, 50)
        c_p_ss4 = command_message.Parameter("languages", c_ri, "japanese")
        c_p_ss5 = command_message.Parameter("character", c_ri, "m1")
        c_al1 = command_message.ArgumentList([c_p_ss1,c_p_ss2,c_p_ss3,c_p_ss4,c_p_ss5])
        c_cm1 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis',"set_parameter","",c_al1)
        c_cm1.delay_time = 0

        c_p = command_message.Parameter("", c_ri, "")
        c_al2 = command_message.ArgumentList([c_p])
        c_cm2 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis',"get_parameter","",c_al2)
        c_cm2.delay_time = 0

        c_p_fl1 = command_message.Parameter("detection_threshold", c_ri, 3)
        c_p_fl2 = command_message.Parameter("minimum_interval", c_ri, 2)
        c_al3_1_1 = command_message.ArgumentList([c_p_fl1,c_p_fl2])
        c_cm3_1_1 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceLocalization',"set_parameter","",c_al3_1_1)
        c_cm3_1_1.delay_time = 3000
    
        c_p_n1 = command_message.Parameter("target_position", c_ri, [100.0,200.0])
        c_p_n2 = command_message.Parameter("time_limit", c_ri, 60000)
        c_p_n3 = command_message.Parameter("routing_policy", c_ri, "time priority")
        c_al3_1_2 = command_message.ArgumentList([c_p_n1,c_p_n2,c_p_n3])
        c_cm3_1_2 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:sub02:Navigation',"set_parameter","",c_al3_1_2)
        c_cm3_1_2.delay_time = 4000

        c_p_m1 = command_message.Parameter("line", c_ri, [1000,360])
        c_p_m2 = command_message.Parameter("curve", c_ri, [1000,90])
        c_p_m3 = command_message.Parameter("time", c_ri, 1000)
        c_al3_2 = command_message.ArgumentList([c_p_m1,c_p_m2,c_p_m3])
        c_cm3_2 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:sub03:Move',"set_parameter","",c_al3_2)
        c_cm3_2.delay_time = 5000

        c_b1 = command_message.Branch([c_cm3_1_1,c_cm3_1_2])
        c_b2 = command_message.Branch([c_cm3_2])
        c_cc = command_message.ConcurrentCommands([c_b1,c_b2])
        c_cc.delay_time = 0

        c_p_f1 = command_message.Parameter("target_object_ref", c_ri, "human1")
        c_p_f2 = command_message.Parameter("distance", c_ri, 500)
        c_p_f3 = command_message.Parameter("time_limit", c_ri, 60000)
        c_al4 = command_message.ArgumentList([c_p_f1,c_p_f2,c_p_f3])
        c_cm4 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow',"set_parameter","",c_al4)
        c_cm4.delay_time = 1000    
        
        c_cus1 = command_message.CommandUnitSequence([c_cm1,c_cm2,c_cc,c_cm4])



        i = HRI_Engine_client_sample.IF("http://127.0.0.1:8016")
        res = [
            i.connect(),
            i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonDetection'),
            i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonIdentification'),
            i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis'),
            i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow'),
            i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceDetection'),
            i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceLocalization'),
            i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub02:Navigation'),
            i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub03:PersonDetection'),
            i.bind('urn:x-rois:def:HRIComponent:Kyutech:sub03:Move'),

            i.subscribe("person_detected",""),
            i.subscribe("person_identified",""),
            i.subscribe("face_detected",""),
            i.subscribe("face_localized",""),

            i.execute(c_cus1),
            
            i.get_command_result("main-exe-1","")
        ]
            
        print(res)

        s_e = set()
        completed_command_id = set()
        for x in range(15):
            e = i.get_event()
            s_e.add((e[0][:3],e[1]))
            if e[1] == "completed":
                completed_command_id.add(e[0][0])
            else:
                pass
        
        print("completed_command_id:{}".format(completed_command_id))
        
        command_unit_lists = [command_unit_list1]
        command_ids = set()
        for y in command_unit_lists:
            d = json.loads(y)
            for c in d["command_unit_list"]:
                c_k = list(c.keys())
                if c_k[0] != "branch_list":
                    if c["command_type"] == "set_parameter":
                        command_ids.add(c["command_id"])
                else:
                    for b in c["branch_list"]:
                        for cl in b["command_list"]:
                            if cl["command_type"] == "set_parameter":
                                command_ids.add(cl["command_id"])

        
        print("set_parameter_command_ids:{}".format(command_ids))

    
        res.extend([
            i.disconnect()
        ])

        self.assertEqual(s_e,event_result)
        self.assertEqual(completed_command_id,command_ids)
        return self.assertEqual(res,
                                [
                                RoIS_HRI.ReturnCode_t.OK,
                            
                                RoIS_HRI.ReturnCode_t.OK, # bind:PersonDetection
                                RoIS_HRI.ReturnCode_t.OK, # bind:PersonIdentification
                                RoIS_HRI.ReturnCode_t.OK, # bind:SpeechSynthesis
                                RoIS_HRI.ReturnCode_t.OK, # bind:Follow
                                RoIS_HRI.ReturnCode_t.OK, # bind:FaceDetection
                                RoIS_HRI.ReturnCode_t.OK, # bind:FaceLocalization
                                RoIS_HRI.ReturnCode_t.OK, # bind:Navigation
                                RoIS_HRI.ReturnCode_t.OK, # bind:PersonDetection
                                RoIS_HRI.ReturnCode_t.OK, # bind:Move

                                (RoIS_HRI.ReturnCode_t.OK,"1"), # subscribe:PersonDetection
                                (RoIS_HRI.ReturnCode_t.OK,"2"), # subscribe:PersonIdentification
                                (RoIS_HRI.ReturnCode_t.OK,"3"), # subscribe:FaceDetection
                                (RoIS_HRI.ReturnCode_t.OK,"4"), # subscribe:FaceLocalization

                                (RoIS_HRI.ReturnCode_t.OK,command_unit_list1), # execute1

                                (RoIS_HRI.ReturnCode_t.OK,["Execute(main-exe-1): Success"]), # get_command_result
                                
                                RoIS_HRI.ReturnCode_t.OK # disconnect
                                ])


    def tearDown(self):
        """ tearDown
        """
        # terminate the server process
        for x in self.ps:
            if x.is_alive():
                x.terminate()

        time.sleep(1)


if __name__ == '__main__':
    # ip = sys.argv
    # if len(ip) > 1:
    #     m = re.search(r'(\d+)\.(\d+)\.(\d+)\.(\d+)',ip[1])
    # else:
    #     m = None
    # if m and len(m.groups()) == 4:
    #     ser_uri = "http://" + ip[1] +":8000"
    #     print("server_url == " + ser_uri + "\n")
    #     del sys.argv[1]
    # else:
    #     ser_uri = "http://127.0.0.1:8000"
    #     print("server_url == " + ser_uri + "\n")
    #     if len(ip) > 1 and ip[1][0] != '-':
    #         del sys.argv[1]

    unittest.main()


# Service_Application_IF.py
#
# Copyright 2017 Eiichi Inohira
# This software may be modified and distributed under the terms
# of the MIT license
#
# for python 3
# for Service Application

"""Service_Application_IF_for_Sub_HRI_Engine
"""


import xmlrpc.client
import threading
import logging
import logging.handlers

import queue
import time

from pyrois import RoIS_Service,RoIS_Common

class Service_Application_IF(RoIS_Service.Service_Application_Base):
    """Service_Application_IF
    """
    def __init__(self, uri, logger=None, event_queue=None):
        self._uri_sa = uri
        self._proxy_sa = xmlrpc.client.ServerProxy(self._uri_sa)
        self.logger = logger
        if event_queue is not None:
            self.event_q = event_queue
        else:
            self.event_q = queue.Queue()
        self.t = threading.Thread(target=self.poll_event,daemon=True)
        self.t.start()

        self.logger.info("Service_Application_IF_sample.Service_Application_IF(): complete init")


    def poll_event(self):
        """poll_event
        """
        # print("sa: poll_event")
        while True:
            try:
                msg = self._proxy_sa.poll_event()
                # print("sa: try poll_event")
            except ConnectionRefusedError:
                # print("sa: error poll_event")
                self.logger.error("poll_event: xmlrpc error")
                return
            (params, methodname) = xmlrpc.client.loads(msg)
            self.logger.info('poll_event: '+methodname)
            if methodname == 'completed' and len(params) == 2:
                self.completed(*params)
            elif methodname == 'notify_error' and len(params) == 2:
                self.notify_error(*params)
            elif methodname == 'notify_event' and len(params) == 4:
                self.notify_event(*params)
            else:
                self.logger.error("poll_event: received unknown event or wrong number of parameters")
    
    def get_event(self,timeout=None):
        (params, methodname) = self.event_q.get(timeout=timeout)
        return (params, methodname)

    def completed(self, command_id, status):
        s = RoIS_Service.Completed_Status(status)
        self.event_q.put(((command_id, s), 'completed'))
        self.logger.info('received completed event: {} {}'.format(command_id, RoIS_Service.Completed_Status(status).name))

    def notify_error(self, error_id, error_type):
        e = RoIS_Service.ErrorType(error_type)
        self.event_q.put(((error_id, e), 'notify_error'))
        self.logger.info('received error event: {} {}'.format(error_id, RoIS_Service.ErrorType(error_type).name))

    def notify_event(self, event_id, event_type, subscribe_id, expire):
        self.event_q.put(((event_id, event_type, subscribe_id, expire), 'notify_event'))
        self.logger.info('received event: {} {} {} {}'.format(event_id,event_type,subscribe_id,expire))


# def example_sa_IF(url, q):
#     try:
#         logger = logging.getLogger('Service_Application_IF_for_sub_HRI_engine')
#         logger.setLevel(logging.DEBUG)
#         # ch = logging.handlers.QueueHandler(q)
#         # ch = logging.StreamHandler()
#         ch = logging.FileHandler('event_notification_test.log')
#         ch.setLevel(logging.DEBUG)
#         formatter = logging.Formatter(
#             '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
#         ch.setFormatter(formatter)
#         logger.addHandler(ch)
#         a = Service_Application_IF(url, logger=logger)
#         time.sleep(20)
#     except KeyboardInterrupt:
#         print("Interrupted")


# if __name__ == '__main__':
#     q = queue.Queue()
#     example_sa_IF('http://127.0.0.1:8016',q)
#     time.sleep(60)

